<?php

namespace Drupal\layout_builder_section_variations\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Alters the layout builder section form to add variation options.
 */
final class AlterLayoutBuilderConfigureSectionForm {

  use StringTranslationTrait;

  /**
   * Constructor for the form alteration class.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service used to interact with storage and entity
   *   loading.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager
  ) {}

  /**
   * Alters the layout builder section configuration form.
   *
   * @param array $form
   *   The form array being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state associated with the form.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function alterForm(
    array &$form,
    FormStateInterface $form_state
  ): void {
    $form_object = $form_state->getFormObject();
    $config = $form_object->getCurrentLayout()->getConfiguration() ?? [];

    $form['layout_settings']['variation'] = [
      '#type' => 'select',
      '#title' => $this->t('Variation'),
      '#options' => $this->listVariationOptions(),
      '#default_value' => $config['layout_builder_section_variations']['variation'] ?? 'default',
    ];

    array_unshift($form['#submit'], [self::class, 'handleSubmit']);
  }

  /**
   * Handles the submission of the layout section configuration form.
   *
   * @param array $form
   *   The submitted form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state associated with the form.
   */
  public static function handleSubmit(
    array $form,
    FormStateInterface $form_state
  ): void {
    $form_object = $form_state->getFormObject();
    $config = $form_object->getCurrentLayout()->getConfiguration() ?? [];

    $config['layout_builder_section_variations'] = [
      'variation' => $form_state->getValue('layout_settings')['variation'] ?? NULL,
    ];

    $form_object->getCurrentLayout()->setConfiguration($config);
  }

  /**
   * Lists available layout variation options.
   *
   * @return array
   *   An array of available variations keyed by their ID.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function listVariationOptions(): array {
    $variations = $this->entityTypeManager
      ->getStorage('section_variation')
      ->loadMultiple();

    $options = [];
    foreach ($variations as $variation) {
      $options[$variation->id()] = $variation->label();
    }

    return $options;
  }

}
