<?php

namespace Drupal\layout_builder_section_variations\Form;

use Drupal\Core\Entity\EntityConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a confirmation form for deleting a Section Variation entity.
 */
final class SectionVariationDeleteForm extends EntityConfirmFormBase {

  use StringTranslationTrait;

  /**
   * Constructs a new SectionVariationDeleteForm.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(MessengerInterface $messenger) {
    $this->messenger = $messenger;
  }

  /**
   * Dependency injection for creating the delete form.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   Returns the instance of the delete form.
   *
   * @noinspection PhpParamsInspection
   */
  public static function create(ContainerInterface $container): self {
    return new self($container->get('messenger'));
  }

  /**
   * Returns the confirmation question.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The question string.
   */
  #[\Override] public function getQuestion(): TranslatableMarkup {
    return $this->t('Are you sure you want to delete %name?', [
      '%name' => $this->entity->label(),
    ]);
  }

  /**
   * Returns the cancel URL.
   *
   * @return \Drupal\Core\Url
   *   The URL to the section variation collection.
   */
  #[\Override] public function getCancelUrl(): Url {
    return new Url('entity.section_variation.collection');
  }

  /**
   * Returns the confirmation button text.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The confirmation button text.
   */
  #[\Override] public function getConfirmText(): TranslatableMarkup {
    return $this->t('Delete');
  }

  /**
   * Form submission handler for deleting the entity.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   *   Throws an exception if entity deletion fails.
   */
  #[\Override] public function submitForm(
    array &$form,
    FormStateInterface $form_state
  ): void {
    $this->entity->delete();

    $this->messenger()->addMessage(
      $this->t('Entity %label has been deleted.',
        ['%label' => $this->entity->label()]
      ));

    $form_state->setRedirectUrl($this->getCancelUrl());
  }

}
