<?php

namespace Drupal\layout_builder_section_variations\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a form for creating or editing a Section Variation entity.
 */
final class SectionVariationForm extends EntityForm {

  use StringTranslationTrait;

  /**
   * Constructs a new SectionVariationForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    MessengerInterface $messenger
  ) {
    $this->entityTypeManager = $entityTypeManager;
    $this->messenger = $messenger;
  }

  /**
   * Dependency injection for creating the form.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   Returns the instance of the SectionVariationForm.
   *
   * @noinspection PhpParamsInspection
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('entity_type.manager'),
      $container->get('messenger')
    );
  }

  /**
   * Builds the form for adding or editing a Section Variation entity.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form structure.
   */
  #[\Override] function form(
    array $form,
    FormStateInterface $form_state
  ): array {
    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t("Label for the Section Variation."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => [$this, 'exist'],
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    return $form;
  }

  /**
   * Handles the form submission to save the entity.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   *   Throws an exception if saving the entity fails.
   */
  #[\Override] public function save(
    array $form,
    FormStateInterface $form_state
  ): void {
    $example = $this->entity;
    $status = $example->save();

    $this->messenger->addMessage(match ($status) {
      SAVED_NEW => $this->t('The %label Example created.', ['%label' => $example->label()]),
      SAVED_UPDATED => $this->t('The %label Example updated.', ['%label' => $example->label()]),
    });

    $form_state->setRedirect('entity.section_variation.collection');
  }

  /**
   * Checks if a Section Variation entity with the given ID already exists.
   *
   * @param string $id
   *   The machine name ID to check.
   *
   * @return bool
   *   TRUE if the entity with the given ID exists, FALSE otherwise.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   *   Thrown if the entity type is not defined.
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *   Thrown if the entity storage is not found.
   */
  public function exist(string $id): bool {
    return (bool) $this->entityTypeManager
      ->getStorage('section_variation')
      ->getQuery()
      ->condition('id', $id)
      ->execute();
  }

}
